# Hunyuan Video GPU 메모리 모니터링 및 최적화 스크립트

import os
import subprocess
import sys
import time
import psutil
import torch

def check_gpu_memory():
    """GPU 메모리 사용량을 확인합니다."""
    if torch.cuda.is_available():
        for i in range(torch.cuda.device_count()):
            print(f"GPU {i}: {torch.cuda.get_device_name(i)}")
            total_memory = torch.cuda.get_device_properties(i).total_memory / 1024**3
            allocated_memory = torch.cuda.memory_allocated(i) / 1024**3
            cached_memory = torch.cuda.memory_reserved(i) / 1024**3
            
            print(f"  총 메모리: {total_memory:.2f} GB")
            print(f"  할당된 메모리: {allocated_memory:.2f} GB")
            print(f"  캐시된 메모리: {cached_memory:.2f} GB")
            print(f"  사용 가능한 메모리: {total_memory - allocated_memory:.2f} GB")
    else:
        print("CUDA를 사용할 수 없습니다.")

def optimize_for_low_vram():
    """저용량 VRAM에 최적화된 환경 설정"""
    # 환경 변수 설정
    os.environ['PYTORCH_CUDA_ALLOC_CONF'] = 'max_split_size_mb:128'
    os.environ['CUDA_LAUNCH_BLOCKING'] = '1'
    
    # 토치 설정
    if torch.cuda.is_available():
        torch.cuda.empty_cache()
        torch.backends.cudnn.benchmark = True
        # 메모리 효율적인 설정
        torch.backends.cuda.matmul.allow_tf32 = True
        torch.backends.cudnn.allow_tf32 = True
        
        print("저용량 VRAM 최적화 적용 완료")
    else:
        print("CUDA를 사용할 수 없어 최적화를 적용할 수 없습니다.")

def check_system_requirements():
    """시스템 요구사항 확인"""
    print("시스템 요구사항 확인:")
    
    # Python 버전 확인
    python_version = sys.version.split()[0]
    print(f"Python 버전: {python_version}")
    if int(python_version.split('.')[0]) < 3 or int(python_version.split('.')[1]) < 10:
        print("  ⚠️ 경고: Python 3.10 이상 필요")
    
    # GPU 확인
    if torch.cuda.is_available():
        print(f"GPU: {torch.cuda.get_device_name(0)}")
        cuda_version = torch.version.cuda
        print(f"CUDA 버전: {cuda_version}")
    else:
        print("⚠️ 경고: NVIDIA GPU가 감지되지 않았습니다")
    
    # 메모리 확인
    ram = psutil.virtual_memory()
    print(f"시스템 메모리: {ram.total / (1024**3):.2f} GB")
    if ram.total < 16 * (1024**3):
        print("  ⚠️ 경고: 16GB 이상의 RAM 권장")
    
    # 디스크 공간 확인
    disk = psutil.disk_usage('/')
    print(f"디스크 공간: {disk.free / (1024**3):.2f} GB 사용 가능")
    if disk.free < 10 * (1024**3):
        print("  ⚠️ 경고: 최소 10GB의 여유 공간 필요")

def install_optimized_dependencies():
    """최적화된 의존성 설치"""
    try:
        # PyTorch 버전 확인
        torch_version = torch.__version__
        cuda_version = torch.version.cuda
        
        print(f"현재 PyTorch 버전: {torch_version}, CUDA 버전: {cuda_version}")
        
        # GPU 메모리 확인
        if torch.cuda.is_available():
            gpu_memory = torch.cuda.get_device_properties(0).total_memory / (1024**3)
            print(f"GPU 메모리: {gpu_memory:.2f} GB")
            
            # 메모리 기반 최적화 패키지 설치
            if gpu_memory < 12:
                print("저용량 GPU 감지: 메모리 최적화 패키지 설치 중...")
                subprocess.check_call([sys.executable, "-m", "pip", "install", "flash-attn==2.7.2.post1"])
                subprocess.check_call([sys.executable, "-m", "pip", "install", "xformers==0.0.29"])
            elif gpu_memory < 24:
                print("중간 용량 GPU 감지: 성능 최적화 패키지 설치 중...")
                subprocess.check_call([sys.executable, "-m", "pip", "install", "flash-attn==2.7.2.post1"])
                subprocess.check_call([sys.executable, "-m", "pip", "install", "sageattention==1.0.6"])
            else:
                print("고용량 GPU 감지: 고성능 최적화 패키지 설치 중...")
                # Sage Attention 2 설치 (예시)
                subprocess.check_call(["git", "clone", "https://github.com/thu-ml/SageAttention"])
                os.chdir("SageAttention")
                subprocess.check_call([sys.executable, "-m", "pip", "install", "-e", "."])
                os.chdir("..")
                
            print("최적화 의존성 설치 완료")
        else:
            print("CUDA를 사용할 수 없어 최적화 패키지를 설치할 수 없습니다.")
    except Exception as e:
        print(f"의존성 설치 중 오류 발생: {str(e)}")

if __name__ == "__main__":
    print("Hunyuan Video 시스템 진단 및 최적화 도구")
    print("=" * 50)
    
    check_system_requirements()
    print("\n" + "=" * 50 + "\n")
    
    check_gpu_memory()
    print("\n" + "=" * 50 + "\n")
    
    choice = input("최적화된 의존성을 설치하시겠습니까? (y/n): ")
    if choice.lower() == 'y':
        install_optimized_dependencies()
    
    choice = input("저용량 VRAM 최적화를 적용하시겠습니까? (y/n): ")
    if choice.lower() == 'y':
        optimize_for_low_vram()
    
    print("\n모든 작업이 완료되었습니다.")
